import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Popover } from 'antd'
import { ExpandOutlined, ReloadOutlined, EllipsisOutlined, LogoutOutlined, ToolOutlined, ScanOutlined, LeftOutlined, EditOutlined, FontColorsOutlined, DeleteOutlined, SearchOutlined, MenuOutlined } from '@ant-design/icons'

import asyncIconComponent from '@/utils/asyncIconComponent'
import getWrapForm from './options'
import MKEmitter from '@/utils/events.js'
import avatar from '@/assets/img/avatar.jpg'
import './index.scss'

const NormalForm = asyncIconComponent(() => import('@/components/normalform'))
const CopyComponent = asyncIconComponent(() => import('@/menu/components/share/copycomponent'))

class NormalNavbar extends Component {
  static propTpyes = {
    card: PropTypes.object,
    deletecomponent: PropTypes.func,
    updateConfig: PropTypes.func,
  }

  state = {
    avatar: sessionStorage.getItem('CloudAvatar') || avatar,
    card: null,
    back: false
  }

  UNSAFE_componentWillMount () {
    const { card } = this.props

    if (card.isNew) {
      let _card = {
        uuid: card.uuid,
        type: card.type,
        width: 24,
        subtype: card.subtype,
        wrap: { type: 'navbar', height: 50, title: 'NavBar', back: 'true', logout: 'false' },
        style: {boxShadow: '0 0 3px #D9D9D9', shadowColor: '#D9D9D9', shadowBlur: '3px', paddingLeft: '10px', paddingRight: '10px', lineHeight: '2.8', fontSize: '18px' },
        searchStyle: {},
        titleStyle: {}
      }

      if (card.config) {
        let config = fromJS(card.config).toJS()

        _card.wrap = config.wrap
        _card.style = config.style
      }

      this.setState({
        card: _card
      })
      this.props.updateConfig(_card)
    } else {
      let _card = fromJS(card).toJS()
      if (_card.wrap.type === 'navbar' && _card.wrap.search === 'true') {
        _card.wrap.type = 'searchIcon'
      }
      this.setState({
        card: _card
      })
    }
  }

  componentDidMount () {
    MKEmitter.addListener('submitSearch', this.getSearch)
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  /**
   * @description 组件销毁，清除state更新，清除快捷键设置
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('submitSearch', this.getSearch)
  }
  
  /**
   * @description 卡片行外层信息更新（数据源，样式等）
   */
  updateComponent = (component) => {
    this.setState({
      card: component
    })

    this.props.updateConfig(component)
  }

  getStyle = (style) => {
    let _card = {...this.state.card, style}

    this.setState({
      card: _card
    })
    
    this.props.updateConfig(_card)
  }

  getSearchStyle = (style) => {
    let _card = {...this.state.card, searchStyle: style}

    this.setState({
      card: _card
    })
    
    this.props.updateConfig(_card)
  }

  getTitleStyle = (style) => {
    let _card = {...this.state.card, titleStyle: style}

    this.setState({
      card: _card
    })
    
    this.props.updateConfig(_card)
  }

  changeStyle = () => {
    const { card } = this.state

    MKEmitter.emit('changeStyle', ['font', 'background', 'border', 'padding', 'shadow'], card.style, this.getStyle)
  }

  changeTitleStyle = () => {
    const { card } = this.state

    MKEmitter.emit('changeStyle', ['font'], card.titleStyle || {}, this.getTitleStyle)
  }

  changeSearchStyle = () => {
    const { card } = this.state

    MKEmitter.emit('changeStyle', ['font', 'background', 'border'], card.searchStyle || {}, this.getSearchStyle)
  }

  getSearch = (config) => {
    const { card } = this.state

    if (card.uuid !== config.uuid) return

    this.setState({
      card: config
    })
    
    this.props.updateConfig(config)
  }

  setSearch = () => {
    let card = fromJS(this.state.card).toJS()

    if (!card.search) {
      card.search = {
        floor: 1,
        setting: { type: 'title', field: '', title: '', focus: 'true', btn: 'hidden', backgroundColor: sessionStorage.getItem('sysBgColor') },
        groups: [],
        fields: []
      }
    }
    this.props.updateConfig(card)
    MKEmitter.emit('changeSearch', card)
  }

  getWrapForms = () => {
    const { wrap } = this.state.card

    return getWrapForm(wrap)
  }

  updateWrap = (res) => {
    let funs = res.funs || []

    delete res.funs

    funs.forEach(n => {
      res[n] = 'true'
    })

    this.updateComponent({...this.state.card, wrap: res})
  }

  skip = () => {
    const { wrap } = this.state.card

    if (!wrap.linkmenu) return
    
    MKEmitter.emit('changeEditMenu', {
      MenuID: wrap.linkmenu,
      copyMenuId: '',
      MenuNo: '',
      MenuName: '',
    })
  }

  render() {
    const { card, avatar } = this.state

    let right = null
    if (card.wrap.logout === 'true') {
      right = <LogoutOutlined />
    }
    if (card.wrap.scan === 'true') {
      right = !right ? <ExpandOutlined onDoubleClick={this.skip}/> : <EllipsisOutlined />
    }
    if (card.wrap.refresh === 'true') {
      right = !right ? <ReloadOutlined /> : <EllipsisOutlined onDoubleClick={this.skip}/>
    }
    if (card.wrap.menus && card.wrap.menus.length > 0) {
      right = !right ? <MenuOutlined /> : <EllipsisOutlined onDoubleClick={this.skip}/>
    }

    return (
      <div className="normal-topbar-edit-box" style={card.style} id={card.uuid}>
        <Popover overlayClassName="mk-popover-control-wrap" mouseLeaveDelay={0.2} mouseEnterDelay={0.2} content={
          <div className="mk-popover-control">
            <NormalForm title="导航栏设置" width={900} update={this.updateWrap} getForms={this.getWrapForms}>
              <EditOutlined style={{color: '#1890ff'}} title="编辑"/>
            </NormalForm>
            <CopyComponent type="topbar" card={card}/>
            <FontColorsOutlined className="style" title="调整样式" onClick={this.changeStyle}/>
            <DeleteOutlined className="close" title="删除组件" onClick={() => this.props.deletecomponent(card.uuid)} />
          </div>
        } trigger="hover">
          <ToolOutlined />
        </Popover>
        <div className="am-navbar">
          <div className="am-navbar-left">
            {card.wrap.back === 'true' && card.wrap.menuPosition !== 'left' ? <LeftOutlined /> : null}
            {card.wrap.menuPosition === 'left' ? <div className="img" style={{backgroundImage: `url(${avatar})`}}></div> : null}
          </div>
          {card.wrap.type !== 'search' ?
            <Popover overlayClassName="mk-popover-control-wrap" mouseLeaveDelay={0.2} mouseEnterDelay={0.2} content={
              <div className="mk-popover-control">
                <FontColorsOutlined className="style" title="调整样式" onClick={this.changeTitleStyle}/>
              </div>
            } trigger="hover">
              <div className="am-navbar-title" style={card.titleStyle}>{card.wrap.title || ''}</div>
            </Popover> :
            <div className="am-navbar-search" onDoubleClick={this.setSearch}>
              <Popover overlayClassName="mk-popover-control-wrap" mouseLeaveDelay={0.2} mouseEnterDelay={0.2} content={
                <div className="mk-popover-control">
                  <FontColorsOutlined className="style" title="调整样式" onClick={this.changeSearchStyle}/>
                </div>
              } trigger="hover">
                <div style={card.searchStyle} className="search-bar">
                  {card.search && card.search.setting.scan === 'true' ? <ScanOutlined /> : <SearchOutlined />}
                </div>
              </Popover>
            </div>
          }
          <div className="am-navbar-right">
            {card.wrap.type === 'searchIcon' ? <SearchOutlined onDoubleClick={this.setSearch}/> : null}
            {right}
          </div>
        </div>
      </div>
    )
  }
}

export default NormalNavbar